@@ -46,6 +46,13 @@
  * gtk_container_add (GTK_CONTAINER (info), scrolled_window);
  *   </programlisting>
  * </example>
+ *
+ * If the calltip is displayed on top of a certain widget, say a #GtkTextView,
+ * you should hide it when the #GtkWidget::focus-out-event signal is emitted by
+ * the #GtkTextView. You may also be interested by the
+ * #GtkTextBuffer:cursor-position property (when its value is modified). If you
+ * use the #GtkSourceCompletionInfo through the #GtkSourceCompletion machinery,
+ * you don't need to worry about this.
  */
 
 #include <gtksourceview/gtksourcecompletioninfo.h>
@@ -80,6 +87,8 @@ idle_resize (GtkSourceCompletionInfo *info)
 	guint border_width;
 	gint window_width;
 	gint window_height;
+	gint cur_window_width;
+	gint cur_window_height;
 
 	g_assert (child != NULL);
 
@@ -92,9 +101,15 @@ idle_resize (GtkSourceCompletionInfo *info)
 	window_width = nat_size.width + 2 * border_width;
 	window_height = nat_size.height + 2 * border_width;
 
-	gtk_window_resize (GTK_WINDOW (info),
-			   MAX (1, window_width),
-			   MAX (1, window_height));
+	gtk_window_get_size (GTK_WINDOW (info), &cur_window_width, &cur_window_height);
+
+	/* Avoid an infinite loop */
+	if (cur_window_width != window_width || cur_window_height != window_height)
+	{
+		gtk_window_resize (GTK_WINDOW (info),
+				   MAX (1, window_width),
+				   MAX (1, window_height));
+	}
 
 	return FALSE;
 }
