@@ -23,6 +23,7 @@
 #include "gtksourcecompletionmodel.h"
 #include "gtksourcecompletionprovider.h"
 #include "gtksourcecompletionproposal.h"
+#include "gtksourceview-i18n.h"
 
 #define GTK_SOURCE_COMPLETION_MODEL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_COMPLETION_MODEL, GtkSourceCompletionModelPrivate))
 
@@ -446,28 +447,39 @@ tree_model_get_value (GtkTreeModel *tree_model,
 			g_value_set_object (value, completion_proposal);
 			break;
 
-		case GTK_SOURCE_COMPLETION_MODEL_COLUMN_LABEL:
-			if (completion_proposal != NULL)
+		case GTK_SOURCE_COMPLETION_MODEL_COLUMN_MARKUP:
+			if (is_header (proposal_info))
 			{
-				gchar *label = gtk_source_completion_proposal_get_label (completion_proposal);
-				g_value_take_string (value, label);
+				gchar *name = gtk_source_completion_provider_get_name (completion_provider);
+
+				if (name != NULL)
+				{
+					gchar *escaped = g_markup_escape_text (name, -1);
+					gchar *markup = g_strdup_printf ("<b>%s</b>", escaped);
+					g_value_take_string (value, markup);
+
+					g_free (name);
+					g_free (escaped);
+				}
+				else
+				{
+					gchar *markup = g_strdup_printf ("<b>%s</b>", _("Provider"));
+					g_value_take_string (value, markup);
+				}
 			}
 			else
-			{
-				g_value_take_string (value, NULL);
-			}
-			break;
-
-		case GTK_SOURCE_COMPLETION_MODEL_COLUMN_MARKUP:
-			if (completion_proposal != NULL)
 			{
 				gchar *markup = gtk_source_completion_proposal_get_markup (completion_proposal);
+
+				if (markup == NULL)
+				{
+					gchar *label = gtk_source_completion_proposal_get_label (completion_proposal);
+					markup = g_markup_escape_text (label != NULL ? label : "", -1);
+					g_free (label);
+				}
+
 				g_value_take_string (value, markup);
 			}
-			else
-			{
-				g_value_take_string (value, NULL);
-			}
 			break;
 
 		case GTK_SOURCE_COMPLETION_MODEL_COLUMN_ICON:
@@ -483,6 +495,10 @@ tree_model_get_value (GtkTreeModel *tree_model,
 			}
 			break;
 
+		case GTK_SOURCE_COMPLETION_MODEL_COLUMN_IS_HEADER:
+			g_value_set_boolean (value, is_header (proposal_info));
+			break;
+
 		default:
 			g_assert_not_reached ();
 	}
@@ -683,11 +699,11 @@ gtk_source_completion_model_init (GtkSourceCompletionModel *self)
 {
 	self->priv = GTK_SOURCE_COMPLETION_MODEL_GET_PRIVATE (self);
 
-	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_PROVIDER] = G_TYPE_OBJECT;
-	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_PROPOSAL] = G_TYPE_OBJECT;
-	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_LABEL] = G_TYPE_STRING;
 	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_MARKUP] = G_TYPE_STRING;
 	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_ICON] = GDK_TYPE_PIXBUF;
+	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_PROPOSAL] = G_TYPE_OBJECT;
+	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_PROVIDER] = G_TYPE_OBJECT;
+	self->priv->column_types[GTK_SOURCE_COMPLETION_MODEL_COLUMN_IS_HEADER] = G_TYPE_BOOLEAN;
 
 	self->priv->show_headers = 1;
 	self->priv->providers = NULL;
